package src.DrawningObjects;
import src.DirectionType;
import src.Entities.EntityAirplane;
import src.MovementStrategy.DrawningObjectAirplane;
import src.MovementStrategy.IMoveableObject;

import java.awt.*;
import java.util.Random;
public class DrawningAirplane {
    protected EntityAirplane EntityAirplane;
    private int _pictureWidth;
    private int _pictureHeight;
    protected int _startPosX;
    protected int _startPosY;
    protected IDraw DrawningIlluminators;
    private int _airplaneWidth = 200;
    private int _airplaneHeight = 78;
    public EntityAirplane EntityAirplane(){
        return EntityAirplane;
    }
    public DrawningAirplane(int speed, double weight, Color bodyColor, int width, int height){
        if(width <= _airplaneWidth || height <= _airplaneHeight)
            return;
        _startPosY=0;
        _startPosX = 0;
        _pictureWidth = width;
        _pictureHeight = height;
        EntityAirplane = new EntityAirplane(speed, weight, bodyColor);
        DrawningIlluminators = new DrawningIlluminators(_airplaneWidth, _airplaneHeight,_startPosX,_startPosY);
        Random rand = new Random();
        int variant = rand.nextInt(0,3);
        if(variant ==0){
            DrawningIlluminators = new DrawningIlluminators(_airplaneWidth, _airplaneHeight,_startPosX,_startPosY);
        } else{
        if(variant ==1){
            DrawningIlluminators = new DrawningIlluminatorsQuad(_airplaneWidth, _airplaneHeight,_startPosX,_startPosY);
        }
        else if(variant==2){
            DrawningIlluminators = new DrawningIlluminatorsCirc(_airplaneWidth, _airplaneHeight,_startPosX,_startPosY);
        }}
        DrawningIlluminators.ChangeIlluminatorNumb(rand.nextInt(1, 5));
    }
    protected DrawningAirplane(int speed, double weight, Color bodyColor, int width, int height,int airplaneWidth, int airplaneHeight){
        if(width <= _airplaneWidth || height <= _airplaneHeight)
            return;
        _startPosY=0;
        _startPosX = 0;
        _pictureWidth = width;
        _pictureHeight = height;
        _airplaneWidth = airplaneWidth;
        _airplaneHeight = airplaneHeight;
        EntityAirplane = new EntityAirplane(speed, weight, bodyColor);
        DrawningIlluminators = new DrawningIlluminators(_airplaneWidth, _airplaneHeight,_startPosX,_startPosY);
        Random rand = new Random();
        int variant = rand.nextInt(0,3);
        if(variant ==0){
            DrawningIlluminators = new DrawningIlluminators(_airplaneWidth, _airplaneHeight,_startPosX,_startPosY);
        } else{
            if(variant ==1){
                DrawningIlluminators = new DrawningIlluminatorsQuad(_airplaneWidth, _airplaneHeight,_startPosX,_startPosY);
            } else if(variant==2){
                DrawningIlluminators = new DrawningIlluminatorsCirc(_airplaneWidth, _airplaneHeight,_startPosX,_startPosY);
            }
        }
        DrawningIlluminators.ChangeIlluminatorNumb(rand.nextInt(1, 5));
    }
    public void SetPosition(int x, int y){
        if(EntityAirplane == null)
            return;
        _startPosX = x;
        _startPosY = y;
        DrawningIlluminators.ChangeX(_startPosX);
        DrawningIlluminators.ChangeY(_startPosY);
        if (x < 0 || y < 0 || x + _airplaneWidth >= _pictureWidth || y + _airplaneHeight >= _pictureHeight) {
            x = y = 10;
            _startPosX = x;
            _startPosY = y;
        }
    }
    public IMoveableObject GetMoveableObject(){
        return new DrawningObjectAirplane(this);
    }
    public int GetPosX(){return _startPosX;}
    public int GetPosY(){return _startPosY;}
    public int GetWidth(){return _airplaneWidth;}
    public int GetHeight(){return _airplaneHeight;}
    public boolean CanMove(DirectionType direction)
    {
        if (EntityAirplane == null)
            return false;
        boolean can = false;
        switch (direction)
        {
            case Left:
                can = _startPosX - EntityAirplane.Step() >= 0;
                break;
            case Right:
                can = _startPosX + EntityAirplane.Step() + _airplaneWidth< _pictureWidth;
                break;
            case Down:
                can = _startPosY + EntityAirplane.Step() + _airplaneHeight < _pictureHeight;
                break;
            case Up:
                can = _startPosY - EntityAirplane.Step() >= 0;
                break;
        };
        return can;
    }
    public void MoveTransport(DirectionType direction){
        if (!CanMove(direction) || EntityAirplane == null)
            return;
        switch (direction)
        {
            case Left:
                if (_startPosX - EntityAirplane.Step() >= 0)
                    _startPosX -= (int)EntityAirplane.Step();
                else
                    _startPosX = 0;
                break;
            case Up:
                if (_startPosY - EntityAirplane.Step() >= 0)
                    _startPosY -= (int)EntityAirplane.Step();
                else
                    _startPosY = 0;
                break;
            case Right:
                if (_startPosX + EntityAirplane.Step() + _airplaneWidth < _pictureWidth)
                    _startPosX += (int)EntityAirplane.Step();
                else
                    _startPosX = _pictureWidth - _airplaneWidth;
                break;
            case Down:
                if (_startPosY + EntityAirplane.Step() + _airplaneHeight < _pictureHeight)
                    _startPosY += (int)EntityAirplane.Step();
                else
                    _startPosY = _pictureHeight - _airplaneHeight;
                break;
        }
        DrawningIlluminators.ChangeX(_startPosX);
        DrawningIlluminators.ChangeY(_startPosY);
    }
    public void DrawAirplane(Graphics2D g2d){
        if (EntityAirplane == null)
            return;
        g2d.setColor(Color.BLACK);
        g2d.setStroke(new BasicStroke(3));
// корпус
        g2d.setColor(EntityAirplane.BodyColor());
        g2d.fillOval(_startPosX, _startPosY + 25, 180, 30);
        g2d.setColor(Color.BLACK);
        g2d.drawOval(_startPosX, _startPosY + 25, 180, 30);
// крыло
        g2d.setColor(Color.BLACK);
        g2d.fillOval(_startPosX + 70, _startPosY + 35, 80, 10);
// стекла
        g2d.setColor(Color.BLUE);
        int[] curvePointsX =
                {
                        _startPosX + 170,
                        _startPosX + 200,
                        _startPosX + 170,
                };
        int[] curvePointsY =
                {
                        _startPosY + 30,
                        _startPosY + 40,
                        _startPosY + 50,
                };
        g2d.fillPolygon(curvePointsX,curvePointsY,curvePointsY.length);
        g2d.setColor(Color.BLACK);
        g2d.drawPolygon(curvePointsX,curvePointsY,curvePointsY.length);
        g2d.drawLine(_startPosX + 170, _startPosY + 40, _startPosX + 200, _startPosY + 40);
// хвост
        g2d.setColor(Color.BLUE);
        int[] curvePoints2x =
                {
                        _startPosX,
                        _startPosX,
                        _startPosX + 30,
                };
        int[] curvePoints2y =
                {
                        _startPosY + 35,
                        _startPosY + 5,
                        _startPosY + 35,
                };
        g2d.fillPolygon(curvePoints2x,curvePoints2y, curvePoints2x.length);
        g2d.setColor(Color.BLACK);
        g2d.drawPolygon(curvePoints2x,curvePoints2y, curvePoints2x.length);
// шасси
        g2d.setColor(Color.BLACK);
        g2d.drawLine(_startPosX + 50, _startPosY + 55, _startPosX + 50, _startPosY + 70);
        g2d.drawLine(_startPosX + 150, _startPosY + 51, _startPosX + 150, _startPosY + 70);
        g2d.fillOval(_startPosX + 40, _startPosY + 65, 10, 10);
        g2d.fillOval(_startPosX + 50, _startPosY + 65, 10, 10);
        g2d.fillOval(_startPosX + 145, _startPosY + 65, 10, 10);
        DrawningIlluminators.DrawIlluminators(g2d);
    }
}
