package src;

import src.DrawningObjects.DrawningAirplane;
import src.Generics.AirplaneGenericCollection;
import src.MovementStrategy.DrawningObjectAirplane;
import src.Generics.AirplaneGenericStorage;
import src.Generics.AirplaneTrashCollection;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;


public class FormAirplaneCollection {
    private final AirplaneGenericStorage _storage;
    private JList<String> listBoxStorages;
    private DefaultListModel<String> listBoxModel;
    private int pictureBoxWidth = 630;
    private int pictureBoxHeight = 426;
    CollectionCanvas canv;
    void Draw(){
        if(canv == null)
            return;
        canv.repaint();
    }
    private void ReloadObjects(){
        int index = listBoxStorages.getSelectedIndex();
        listBoxModel.clear();
        List<String> keys = _storage.Keys();
        for(int i = 0; i < keys.size(); i++){
            listBoxModel.addElement(keys.get(i));
        }
        if(listBoxModel.size() > 0 && (index == -1 || index >= listBoxModel.size()))
            listBoxStorages.setSelectedIndex(0);
        else if(listBoxModel.size() > 0)
            listBoxStorages.setSelectedIndex(index);
    }

    public FormAirplaneCollection(){
        AirplaneTrashCollection<DrawningAirplane> _trashCollection = new AirplaneTrashCollection<>();
        JButton callTrashButton = new JButton("Мусор");
        _storage = new AirplaneGenericStorage(pictureBoxWidth, pictureBoxHeight);
        JScrollPane scrollPane = new JScrollPane();
        canv = new CollectionCanvas();
        JPanel toolBox = new JPanel();
        JTextField storageName = new JTextField();
        JButton addStorageButton = new JButton("Добавить набор");
        listBoxModel = new DefaultListModel<>();
        listBoxStorages= new JList<>(listBoxModel);
        scrollPane.setViewportView(listBoxStorages);
        JButton delStorageButton = new JButton("Удалить набор");
        //toolBox.setBounds(623,12, 227, 80);
        JFrame collectionFrame = new JFrame();
        collectionFrame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        collectionFrame.setSize(880,497);
        toolBox.setBounds(623, 12, 227, 426);
        canv.setBounds(12,12,pictureBoxWidth,pictureBoxHeight);
        JButton addButton = new JButton("Добавить");
        JButton removeButton = new JButton("Удалить");
        JButton refreshButton = new JButton("Обновить");
        JTextField airplaneNumb = new JTextField();
        GridLayout lay = new GridLayout(9,1);
        toolBox.add(storageName);
        toolBox.add(addStorageButton);
        toolBox.add(scrollPane);
        toolBox.add(delStorageButton);
        toolBox.setLayout(lay);
        toolBox.add(addButton);
        toolBox.add(airplaneNumb);
        toolBox.add(removeButton);
        toolBox.add(callTrashButton);
        toolBox.add(refreshButton);
        collectionFrame.add(toolBox);
        collectionFrame.add(canv);
        collectionFrame.setVisible(true);
        canv._storage = _storage;
        canv.listBoxStorages = listBoxStorages;

        addStorageButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(storageName.getText() == null)
                    return;
                _storage.AddSet(storageName.getText());
                ReloadObjects();
            }
        });

        delStorageButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(listBoxStorages.getSelectedIndex() == -1) {
                    return;
                }
                _storage.DelSet(listBoxStorages.getSelectedValue());
                ReloadObjects();
            }
        });

        callTrashButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(_trashCollection.GetSize() == 0)
                    return;
                FormAirplaneWithRadar form = new FormAirplaneWithRadar();
                form.ChangeAirplane(_trashCollection.GetTop());
                _trashCollection.Pop();
            }
        });

        addButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(listBoxStorages.getSelectedIndex() == -1) {
                    return;
                }
                AirplaneGenericCollection<DrawningAirplane, DrawningObjectAirplane> _airplanes = _storage.Get(listBoxStorages.getSelectedValue());
                FormAirplaneWithRadar form = new FormAirplaneWithRadar();
                form.buttonSelect.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        if (_airplanes.Insert(form.SelectedAirplane()))
                        {
                            JOptionPane.showMessageDialog(null, "Объект добавлен", "Информация", JOptionPane.INFORMATION_MESSAGE);
                            Draw();
                        }
                        else {
                            JOptionPane.showMessageDialog(null, "Не удалось добавить объект", "Информация", JOptionPane.INFORMATION_MESSAGE);
                        }
                        form.AirplaneFrame.dispose();
                        Draw();
                    }
                });
            }
        });
        removeButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(listBoxStorages.getSelectedIndex() == -1) {
                    return;
                }
                AirplaneGenericCollection<DrawningAirplane, DrawningObjectAirplane> _airplanes = _storage.Get(listBoxStorages.getSelectedValue());
                if(_airplanes == null) {
                    return;
                }
                String tmp = airplaneNumb.getText();
                int numb;
                try{
                    numb = Integer.parseInt(tmp);
                }
                catch(Exception ex){
                    JOptionPane.showMessageDialog(null, "Введите число", "Информация", JOptionPane.INFORMATION_MESSAGE);
                    return;
                }
                DrawningAirplane curAirplane = _airplanes.Get(numb);
                _trashCollection.Push(curAirplane);
                _airplanes.Remove(numb);
                _airplanes.ShowAirplanes();
                JOptionPane.showMessageDialog(null, "Объект удален", "Информация", JOptionPane.INFORMATION_MESSAGE);
                Draw();
            }
        });

        refreshButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(listBoxStorages.getSelectedIndex() == -1) {
                    return;
                }
                AirplaneGenericCollection<DrawningAirplane, DrawningObjectAirplane> _airplanes = _storage.Get(listBoxStorages.getSelectedValue());
                if(_airplanes == null) {
                    return;
                }
                _airplanes.ShowAirplanes();
                Draw();
            }
        });
    }
}

class CollectionCanvas extends JComponent {
    public AirplaneGenericStorage _storage;
    public JList<String> listBoxStorages;
    public CollectionCanvas(){
    }
    @Override
    public void paintComponent (Graphics g){
        if (listBoxStorages == null || listBoxStorages.getSelectedIndex() == -1){
            return;
        }
        super.paintComponents (g) ;
        Graphics2D g2d = (Graphics2D)g;
        g2d.drawImage(_storage.Get(listBoxStorages.getSelectedValue()).ShowAirplanes(), 0, 0, this);
        super.repaint();
    }
}
